﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ServiceModel;
using Cbcye.Contract;

namespace Cbcye.Service
{
    #region ConcurrencyMode说明
    /*
     
     * Single: 默认方式。服务实例是 single-threaded，不接受重入调用(reentrant calls)。也就是说对于同一个服务实例的多个调用必须排队，
     * 直到上一次调用完成后才能继续。 
     * 
     * Reentrant: 和 Single 一样，也是 single-threaded，但能接受重入调用，至于针对同一服务对象的多个调用依然需要排队。
     * 在 Single 模式下，当方法调用另外一个服务(Callback是客户端提供的服务)时，方法会阻塞，直到所调用的服务完成。
     * 如果方法不能重入，那么因无法接受所调用服务的返回消息(reply message)，无法解除阻塞状态而陷入死锁(deadlock)。
     * Reentrant 模式就是为了解决 Single 的这种不足，允许方法重入以完成处理过程。 
     * 
     * Multiple: 和 Single、Reentrant 不同，Multiple 允许多个客户端同时调用服务方法。不再有锁的问题，
     * 同样也不再提供同步保障(synchronization guarantees)。使用此模式时，
     * 我们必须自行使用多线程同步机制(如使用 lock 关键字) 来保证数据成员的读写安全。 
     */
    #endregion

    //指示服务模式为多线程模式
    //Multiple 允许多个客户端同时调用服务方法。不再有锁的问题，同样也不再提供同步保障(synchronization guarantees)。
    //使用此模式时，我们必须自行使用多线程同步机制(如使用 lock 关键字) 来保证数据成员的读写安全。
    [ServiceBehavior(ConcurrencyMode=ConcurrencyMode.Multiple)]
    public class DuplexCalculatorService:IDuplexCalculator
    {

        #region IDuplexCalculator Members

        public void Add(double x, double y)
        {
            double result = x + y;
            //调用 GetCallbackChannel<(Of <(T>)>) 属性以获取通道，使用该通道可以调用调用服务的客户端实例的操作。
            ICalculatorCallback callBack = OperationContext.Current.GetCallbackChannel<ICalculatorCallback>();
            callBack.ShowResult(x, y, result);
        }

        #endregion
    }
}
